<?php

declare(strict_types=1);

namespace mirzaev\vk\robot\bobby\programs;

// Фреймворк ВКонтакте
use mirzaev\vk\robots\robot;

// Модуль чат-робота для фреймворка ВКонтакте
use mirzaev\vk\chat\core,
    mirzaev\vk\chat\program;

// Встроенные библиотеки
use Exception;

/**
 * Программа: "ролевое действие"
 *
 * Удаляет запрос и отправляет в чат выполняемое действие.
 * Можно использовать псевдонимы и падежи
 *
 * @package mirzaev\vk\robot\bobby\programs
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class roleplay_action extends program
{
    /**
     * Конструктор
     *
     * @param robot $robot Робот, который будет выполнять команду
     * @param array $accounts Реестр аккаунтов которым разрешено выполнять команду
     */
    public function __construct(
        public robot $robot,
        public array $accounts = []
    ) {
        parent::__construct(function (core $core, robot $robot, array &$update, string $original, string $text = ''): void {
            try {
                // Инициализация длины сообщения
                $original_length = strlen($update['object']['message']['text']);

                // Инициализция длины распознанных команд
                $command_length = strlen($text) + 2; // +2 это два символа "*" - особенность регулярного выражения
                if (!empty($buttons)) $command_length += strlen($buttons);
                if (!empty($parameters)) $command_length += strlen($parameters);

                // Удаление сообщения, если в нём нет лишнего текста
                if ($original_length === $command_length) $robot->message()->delete($update['object']['message']['conversation_message_id'], $update['object']['message']['peer_id']);
            } catch (Exception $e) {
            }

            if (empty($text = trim($text))) {
                // Пустой текст сообщения

                // Инициализация данных аккаунта отправителя
                $sender = $robot->user()->get($update['object']['message']['from_id']);

                // Инициализация сообщения с ролевым действием
                $core->message($robot->message()->destination($update['object']['message']['peer_id'])->text("@id$sender->id ($sender->first_name) тупит")->disable_mentions(true));

                return;
            }

            // Инициализация индикаторов наличия изменённого псевдонима у получателя и отпрвителя
            $sender_changed = $receiver_changed = false;

            // Инициализация псевдонима получателя
            $receiver_alias = '';

            if (isset($update['object']['message']['reply_message'])) {
                // Найдено сообщение для ответа

                // Инициализация данных аккаунта получателя
                $receiver = $robot->user()->get($update['object']['message']['reply_message']['from_id']);

                if (isset($receiver)) {
                    // Найден получатель

                    // Обработка падежа получателя
                    $text = preg_replace_callback('/(*UTF8)^(.*)\|([^$]+)$/', function (array $matches) use ($text, &$receiver_alias, $receiver): string {
                        // Инициализация псевдонима отправителя
                        $alias = preg_replace('/\)/', '&#41;', $matches[2], 300);

                        // Инициализация псевдонима получателя
                        $receiver_alias = " @id$receiver->id ($receiver->first_name" . (empty($alias) ? '' : $alias) . ')';

                        return $matches[1] ?? $matches[0] ?? $text;
                    }, $text, 1);

                    if (empty($receiver_alias)) {
                        // Не найден псевдоним получателя

                        // Обработка псевдонима получателя
                        $text = preg_replace_callback('/(*UTF8)^(.*)\(([^\)]*)\)$/', function (array $matches) use ($text, &$receiver_alias, $receiver): string {
                            // Инициализация псевдонима отправителя
                            $alias = preg_replace('/\)/', '&#41;', $matches[2], 300);

                            // Инициализация псевдонима получателя
                            $receiver_alias = " @id$receiver->id (" . (empty($alias) ? $receiver->first_name : $alias) . ')';

                            return $matches[1] ?? $matches[0] ?? $text;
                        }, $text, 1);
                    }
                }

                // Проверка на пустого получателя
                if (empty(trim($receiver_alias))) {
                    // Пустой текст получателя

                    // Реинициализация текста получателя
                    $receiver_alias = " @id$receiver->id ($receiver->first_name)";
                } else $receiver_changed = true;
            }

            // Инициализация псевдонима отправителя
            $sender_alias = '';

            // Инициализация данных аккаунта отправителя
            $sender = $robot->user()->get($update['object']['message']['from_id']);

            // Обработка падежа отправителя
            $text = preg_replace_callback('/(*UTF8)^\|([^\s]+)(.*)$/', function (array $matches) use ($text, $sender, &$sender_alias): string {
                // Инициализация псевдонима отправителя
                $alias = preg_replace('/\)/', '&#41;', $matches[1], 300);

                // Инициализация ссылки на отправителя
                $sender_alias = "@id$sender->id ($sender->first_name" . (empty($alias) ? '' : $alias) . ') ';

                return $matches[2] ?? $matches[0] ?? $text;
            }, $text, 1);

            if (empty($sender_alias)) {
                // Не найден псевдоним отправителя

                // Обработка псевдонима отправителя
                $text = preg_replace_callback('/(*UTF8)^\(([^\)]*)\)(.*)$/', function (array $matches) use ($text, $sender, &$sender_alias): string {
                    // Инициализация псевдонима отправителя
                    $alias = preg_replace('/\)/', '&#41;', $matches[1], 300);

                    // Инициализация ссылки на отправителя
                    $sender_alias = "@id$sender->id (" . (empty($alias) ? $sender->first_name : $alias) . ') ';

                    return $matches[2] ?? $matches[0] ?? $text;
                }, $text, 1);
            }

            if (empty(trim($sender_alias))) {
                // Пустой текст отправителя

                // Реинициализация текста отправителя
                $sender_alias = "@id$sender->id ($sender->first_name) ";
            } else $sender_changed = true;

            // Удаление команд падежей из строки
            $text = preg_replace('/(*UTF8)\|[А-яё\w]+/', '', $text, 500);

            // Инициализация сообщения с ролевым действием
            if (empty($text = trim($text)) && !$sender_changed && !$receiver_changed) $core->message($robot->message()->destination($update['object']['message']['peer_id'])->text("@id$sender->id ($sender->first_name) тупит")->mentions(false));
            else $core->message($robot->message()->destination($update['object']['message']['peer_id'])->text("$sender_alias$text$receiver_alias")->mentions(false), 'roleplay_action');
        }, $robot, $accounts);
    }
}
